@@ -6,6 +6,7 @@ use std::os::raw::c_int;
 use backtrace::Frame;
 use nix::sys::signal;
 use parking_lot::RwLock;
+use smallvec::SmallVec;
 
 #[cfg(any(target_arch = "x86_64", target_arch = "aarch64"))]
 use findshlibs::{Segment, SharedLibrary, TargetSharedLibrary};
@@ -31,6 +32,7 @@ pub struct Profiler {
     blocklist_segments: Vec<(usize, usize)>,
 }
 
+#[derive(Clone)]
 pub struct ProfilerGuardBuilder {
     frequency: c_int,
     #[cfg(all(any(target_arch = "x86_64", target_arch = "aarch64")))]
@@ -133,7 +135,10 @@ impl ProfilerGuard<'_> {
 
     /// Generate a report
     pub fn report(&self) -> ReportBuilder {
-        ReportBuilder::new(self.profiler)
+        ReportBuilder::new(
+            self.profiler,
+            self.timer.as_ref().map(Timer::timing).unwrap_or_default(),
+        )
     }
 }
 
@@ -192,7 +197,6 @@ fn write_thread_name(current_thread: libc::pthread_t, name: &mut [libc::c_char])
 }
 
 #[no_mangle]
-#[allow(clippy::uninit_assumed_init)]
 #[cfg_attr(
     not(all(any(target_arch = "x86_64", target_arch = "aarch64"))),
     allow(unused_variables)
@@ -240,14 +244,13 @@ extern "C" fn perf_signal_handler(
                 }
             }
 
-            let mut bt: [Frame; MAX_DEPTH] =
-                unsafe { std::mem::MaybeUninit::uninit().assume_init() };
+            let mut bt: SmallVec<[Frame; MAX_DEPTH]> = SmallVec::with_capacity(MAX_DEPTH);
             let mut index = 0;
 
             unsafe {
                 backtrace::trace_unsynchronized(|frame| {
                     if index < MAX_DEPTH {
-                        bt[index] = frame.clone();
+                        bt.push(frame.clone());
                         index += 1;
                         true
                     } else {
@@ -263,7 +266,7 @@ extern "C" fn perf_signal_handler(
             write_thread_name(current_thread, &mut name);
 
             let name = unsafe { std::ffi::CStr::from_ptr(name_ptr) };
-            profiler.sample(&bt[0..index], name.to_bytes(), current_thread as u64);
+            profiler.sample(bt, name.to_bytes(), current_thread as u64);
         }
     }
 }
@@ -344,7 +347,12 @@ impl Profiler {
     }
 
     // This function has to be AS-safe
-    pub fn sample(&mut self, backtrace: &[Frame], thread_name: &[u8], thread_id: u64) {
+    pub fn sample(
+        &mut self,
+        backtrace: SmallVec<[Frame; MAX_DEPTH]>,
+        thread_name: &[u8],
+        thread_id: u64,
+    ) {
         let frames = UnresolvedFrames::new(backtrace, thread_name, thread_id);
         self.sample_counter += 1;
 
@@ -421,8 +429,8 @@ mod tests {
             }
         }
         let mut prime_numbers = vec![];
-        for i in 2..10000 {
-            if prime_number_table[i] {
+        for (i, item) in prime_number_table.iter().enumerate().skip(2) {
+            if *item {
                 prime_numbers.push(i);
             }
         }
@@ -452,7 +460,7 @@ mod tests {
         }
 
         FLAG.with(|flag| {
-            assert_eq!(*flag.borrow(), false);
+            assert!(!*flag.borrow());
         });
     }
 }
